@@ -548,10 +548,11 @@ class Catalog(base.CatalogDriverBase):
         else:
             return endpoint_group_project_ref
 
-    def list_endpoint_groups(self):
+    def list_endpoint_groups(self, hints):
         with sql.session_for_read() as session:
             query = session.query(EndpointGroup)
-            endpoint_group_refs = query.all()
+            endpoint_group_refs = sql.filter_limit_query(
+                EndpointGroup, query, hints)
             return [e.to_dict() for e in endpoint_group_refs]
 
     def list_endpoint_groups_for_project(self, project_id):
