@@ -55,9 +55,12 @@ class Mapping(base.MappingDriverBase):
             except sql.NotFound:
                 return None
 
-    def get_domain_mapping_list(self, domain_id):
+    def get_domain_mapping_list(self, domain_id, entity_type=None):
+        filters = {'domain_id': domain_id}
+        if entity_type is not None:
+            filters['entity_type'] = entity_type
         with sql.session_for_read() as session:
-            return session.query(IDMapping).filter_by(domain_id=domain_id)
+            return session.query(IDMapping).filter_by(**filters)
 
     def get_id_mapping(self, public_id):
         with sql.session_for_read() as session:
