@@ -107,6 +107,42 @@ class ApplicationCredentialTests(object):
                           self.app_cred_api.create_application_credential,
                           app_cred)
 
+    def test_create_application_credential_with_access_rules(self):
+        app_cred = self._new_app_cred_data(self.user_foo['id'],
+                                           project_id=self.project_bar['id'])
+        app_cred['access_rules'] = [{
+            'id': uuid.uuid4().hex,
+            'service': uuid.uuid4().hex,
+            'path': uuid.uuid4().hex,
+            'method': uuid.uuid4().hex[16:]
+        }]
+        resp = self.app_cred_api.create_application_credential(app_cred)
+        resp.pop('roles')
+        resp_access_rules = resp.pop('access_rules')
+        app_cred.pop('roles')
+        orig_access_rules = app_cred.pop('access_rules')
+        self.assertDictEqual(app_cred, resp)
+        for i, ar in enumerate(resp_access_rules):
+            self.assertDictEqual(orig_access_rules[i], ar)
+
+    def test_create_application_credential_with_preexisting_access_rules(self):
+        app_cred_1 = self._new_app_cred_data(self.user_foo['id'],
+                                             project_id=self.project_bar['id'])
+        app_cred_1['access_rules'] = [{
+            'id': uuid.uuid4().hex,
+            'service': uuid.uuid4().hex,
+            'path': uuid.uuid4().hex,
+            'method': uuid.uuid4().hex[16:]
+        }]
+        resp = self.app_cred_api.create_application_credential(app_cred_1)
+        resp_access_rules_1 = resp.pop('access_rules')
+        app_cred_2 = self._new_app_cred_data(self.user_foo['id'],
+                                             project_id=self.project_bar['id'])
+        app_cred_2['access_rules'] = [{'id': resp_access_rules_1[0]['id']}]
+        resp = self.app_cred_api.create_application_credential(app_cred_2)
+        resp_access_rules_2 = resp.pop('access_rules')
+        self.assertDictEqual(resp_access_rules_1[0], resp_access_rules_2[0])
+
     def test_get_application_credential(self):
         app_cred = self._new_app_cred_data(self.user_foo['id'],
                                            project_id=self.project_bar['id'])
@@ -207,11 +243,14 @@ class ApplicationCredentialTests(object):
 
     def test_removing_user_from_project_deletes_application_credentials(self):
         app_cred_proj_A_1 = self._new_app_cred_data(
-            self.user_foo['id'], project_id=self.project_bar['id'], name='app1')
+            self.user_foo['id'], project_id=self.project_bar['id'],
+            name='app1')
         app_cred_proj_A_2 = self._new_app_cred_data(
-            self.user_foo['id'], project_id=self.project_bar['id'], name='app2')
+            self.user_foo['id'], project_id=self.project_bar['id'],
+            name='app2')
         app_cred_proj_B = self._new_app_cred_data(
-            self.user_foo['id'], project_id=self.project_baz['id'], name='app3')
+            self.user_foo['id'], project_id=self.project_baz['id'],
+            name='app3')
         PROVIDERS.assignment_api.add_role_to_user_and_project(
             project_id=self.project_baz['id'],
             user_id=self.user_foo['id'],
@@ -286,3 +325,43 @@ class ApplicationCredentialTests(object):
                           self.app_cred_api.authenticate,
                           resp['id'],
                           badpass)
+
+    def test_get_delete_access_rules(self):
+        app_cred = self._new_app_cred_data(self.user_foo['id'],
+                                           project_id=self.project_bar['id'])
+        access_rule_id = uuid.uuid4().hex
+        app_cred['access_rules'] = [{
+            'id': access_rule_id,
+            'service': uuid.uuid4().hex,
+            'path': uuid.uuid4().hex,
+            'method': uuid.uuid4().hex[16:]
+        }]
+        self.app_cred_api.create_application_credential(app_cred)
+        self.assertDictEqual(app_cred['access_rules'][0],
+                             self.app_cred_api.get_access_rule(access_rule_id))
+        self.app_cred_api.delete_application_credential(app_cred['id'])
+        self.app_cred_api.delete_access_rule(access_rule_id)
+        self.assertRaises(exception.AccessRuleNotFound,
+                          self.app_cred_api.get_access_rule,
+                          access_rule_id)
+
+    def test_list_delete_access_rule_for_user(self):
+        app_cred = self._new_app_cred_data(self.user_foo['id'],
+                                           project_id=self.project_bar['id'])
+        access_rule_id = uuid.uuid4().hex
+        app_cred['access_rules'] = [{
+            'id': access_rule_id,
+            'service': uuid.uuid4().hex,
+            'path': uuid.uuid4().hex,
+            'method': uuid.uuid4().hex[16:]
+        }]
+        self.app_cred_api.create_application_credential(app_cred)
+        self.assertEqual(1, len(self.app_cred_api.list_access_rules_for_user(
+            self.user_foo['id'])))
+        self.app_cred_api.delete_application_credential(app_cred['id'])
+        # access rule should still exist
+        self.assertEqual(1, len(self.app_cred_api.list_access_rules_for_user(
+            self.user_foo['id'])))
+        self.app_cred_api.delete_access_rules_for_user(self.user_foo['id'])
+        self.assertEqual(0, len(self.app_cred_api.list_access_rules_for_user(
+            self.user_foo['id'])))
