@@ -12,6 +12,7 @@
 
 import uuid
 
+from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
 from keystone.tests import unit
@@ -20,6 +21,7 @@ from keystone.tests.unit import test_v3
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class DuplicateTestCase(test_v3.RestfulTestCase):
@@ -34,10 +36,10 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_domain_duplicate_conflict_gives_name(self):
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
         domain['id'] = uuid.uuid4().hex
         try:
-            self.resource_api.create_domain(domain['id'], domain)
+            PROVIDERS.resource_api.create_domain(domain['id'], domain)
         except exception.Conflict as e:
             self.assertIn("%s" % domain['name'], repr(e))
         else:
@@ -45,10 +47,10 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_project_duplicate_conflict_gives_name(self):
         project = unit.new_project_ref(domain_id=self.domain_id)
-        self.resource_api.create_project(project['id'], project)
+        PROVIDERS.resource_api.create_project(project['id'], project)
         project['id'] = uuid.uuid4().hex
         try:
-            self.resource_api.create_project(project['id'], project)
+            PROVIDERS.resource_api.create_project(project['id'], project)
         except exception.Conflict as e:
             self.assertIn("%s" % project['name'], repr(e))
         else:
@@ -56,10 +58,10 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_user_duplicate_conflict_gives_name(self):
         user = unit.new_user_ref(domain_id=CONF.identity.default_domain_id)
-        user = self.identity_api.create_user(user)
+        user = PROVIDERS.identity_api.create_user(user)
         user['id'] = uuid.uuid4().hex
         try:
-            self.identity_api.create_user(user)
+            PROVIDERS.identity_api.create_user(user)
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with name %s" % user['name'],
                           repr(e))
@@ -68,10 +70,10 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_role_duplicate_conflict_gives_name(self):
         role = unit.new_role_ref()
-        self.role_api.create_role(role['id'], role)
+        PROVIDERS.role_api.create_role(role['id'], role)
         role['id'] = uuid.uuid4().hex
         try:
-            self.role_api.create_role(role['id'], role)
+            PROVIDERS.role_api.create_role(role['id'], role)
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with name %s" % role['name'],
                           repr(e))
@@ -80,9 +82,9 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_group_duplicate_conflict_gives_name(self):
         group = unit.new_group_ref(domain_id=CONF.identity.default_domain_id)
-        group = self.identity_api.create_group(group)
+        group = PROVIDERS.identity_api.create_group(group)
         try:
-            self.identity_api.create_group(group)
+            PROVIDERS.identity_api.create_group(group)
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with name %s"
                           % group['name'], repr(e))
@@ -91,9 +93,9 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_policy_duplicate_conflict_gives_name(self):
         policy_ref = unit.new_policy_ref()
-        self.policy_api.create_policy(policy_ref['id'], policy_ref)
+        PROVIDERS.policy_api.create_policy(policy_ref['id'], policy_ref)
         try:
-            self.policy_api.create_policy(policy_ref['id'], policy_ref)
+            PROVIDERS.policy_api.create_policy(policy_ref['id'], policy_ref)
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with name %s"
                           % policy_ref['name'], repr(e))
@@ -103,9 +105,13 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
     def test_credential_duplicate_conflict_gives_name(self):
         user = unit.new_user_ref(domain_id=CONF.identity.default_domain_id)
         credential = unit.new_credential_ref(user_id=user['id'])
-        self.credential_api.create_credential(credential['id'], credential)
+        PROVIDERS.credential_api.create_credential(
+            credential['id'], credential
+        )
         try:
-            self.credential_api.create_credential(credential['id'], credential)
+            PROVIDERS.credential_api.create_credential(
+                credential['id'], credential
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % credential['id'], repr(e))
@@ -114,15 +120,19 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_trust_duplicate_conflict_gives_name(self):
         trustor = unit.new_user_ref(domain_id=self.domain_id)
-        trustor = self.identity_api.create_user(trustor)
+        trustor = PROVIDERS.identity_api.create_user(trustor)
         trustee = unit.new_user_ref(domain_id=self.domain_id)
-        trustee = self.identity_api.create_user(trustee)
+        trustee = PROVIDERS.identity_api.create_user(trustee)
         role_ref = unit.new_role_ref()
-        self.role_api.create_role(role_ref['id'], role_ref)
+        PROVIDERS.role_api.create_role(role_ref['id'], role_ref)
         trust_ref = unit.new_trust_ref(trustor['id'], trustee['id'])
-        self.trust_api.create_trust(trust_ref['id'], trust_ref, [role_ref])
+        PROVIDERS.trust_api.create_trust(
+            trust_ref['id'], trust_ref, [role_ref]
+        )
         try:
-            self.trust_api.create_trust(trust_ref['id'], trust_ref, [role_ref])
+            PROVIDERS.trust_api.create_trust(
+                trust_ref['id'], trust_ref, [role_ref]
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % trust_ref['id'], repr(e))
@@ -132,11 +142,13 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
     def test_mapping_duplicate_conflict_gives_name(self):
         self.mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
         self.mapping['id'] = uuid.uuid4().hex
-        self.federation_api.create_mapping(self.mapping['id'],
-                                           self.mapping)
+        PROVIDERS.federation_api.create_mapping(
+            self.mapping['id'], self.mapping
+        )
         try:
-            self.federation_api.create_mapping(self.mapping['id'],
-                                               self.mapping)
+            PROVIDERS.federation_api.create_mapping(
+                self.mapping['id'], self.mapping
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % self.mapping['id'], repr(e))
@@ -146,11 +158,13 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
     def test_mapping_duplicate_conflict_with_id_in_id(self):
         self.mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
         self.mapping['id'] = 'mapping_with_id_in_the_id'
-        self.federation_api.create_mapping(self.mapping['id'],
-                                           self.mapping)
+        PROVIDERS.federation_api.create_mapping(
+            self.mapping['id'], self.mapping
+        )
         try:
-            self.federation_api.create_mapping(self.mapping['id'],
-                                               self.mapping)
+            PROVIDERS.federation_api.create_mapping(
+                self.mapping['id'], self.mapping
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % self.mapping['id'], repr(e))
@@ -158,9 +172,9 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
 
     def test_region_duplicate_conflict_gives_name(self):
         region_ref = unit.new_region_ref()
-        self.catalog_api.create_region(region_ref)
+        PROVIDERS.catalog_api.create_region(region_ref)
         try:
-            self.catalog_api.create_region(region_ref)
+            PROVIDERS.catalog_api.create_region(region_ref)
         except exception.Conflict as e:
             self.assertIn("Duplicate ID, %s" % region_ref['id'], repr(e))
         else:
@@ -172,22 +186,23 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
             'enabled': True,
             'description': uuid.uuid4().hex
         }
-        self.federation_api.create_idp(self.idp['id'], self.idp)
+        PROVIDERS.federation_api.create_idp(self.idp['id'], self.idp)
         self.mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
         self.mapping['id'] = uuid.uuid4().hex
-        self.federation_api.create_mapping(self.mapping['id'],
-                                           self.mapping)
+        PROVIDERS.federation_api.create_mapping(
+            self.mapping['id'], self.mapping
+        )
         protocol = {
             'id': uuid.uuid4().hex,
             'mapping_id': self.mapping['id']
         }
-        protocol_ret = self.federation_api.create_protocol(self.idp['id'],
-                                                           protocol['id'],
-                                                           protocol)
+        protocol_ret = PROVIDERS.federation_api.create_protocol(
+            self.idp['id'], protocol['id'], protocol
+        )
         try:
-            self.federation_api.create_protocol(self.idp['id'],
-                                                protocol['id'],
-                                                protocol)
+            PROVIDERS.federation_api.create_protocol(
+                self.idp['id'], protocol['id'], protocol
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % protocol_ret['id'], repr(e))
@@ -201,22 +216,23 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
             'enabled': True,
             'description': uuid.uuid4().hex
         }
-        self.federation_api.create_idp(self.idp['id'], self.idp)
+        PROVIDERS.federation_api.create_idp(self.idp['id'], self.idp)
         self.mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
         self.mapping['id'] = uuid.uuid4().hex
-        self.federation_api.create_mapping(self.mapping['id'],
-                                           self.mapping)
+        PROVIDERS.federation_api.create_mapping(
+            self.mapping['id'], self.mapping
+        )
         protocol = {
             'id': 'federation_protocol_with_id_in_the_id',
             'mapping_id': self.mapping['id']
         }
-        protocol_ret = self.federation_api.create_protocol(self.idp['id'],
-                                                           protocol['id'],
-                                                           protocol)
+        protocol_ret = PROVIDERS.federation_api.create_protocol(
+            self.idp['id'], protocol['id'], protocol
+        )
         try:
-            self.federation_api.create_protocol(self.idp['id'],
-                                                protocol['id'],
-                                                protocol)
+            PROVIDERS.federation_api.create_protocol(
+                self.idp['id'], protocol['id'], protocol
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % protocol_ret['id'], repr(e))
@@ -228,22 +244,23 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
             'enabled': True,
             'description': uuid.uuid4().hex
         }
-        self.federation_api.create_idp(self.idp['id'], self.idp)
+        PROVIDERS.federation_api.create_idp(self.idp['id'], self.idp)
         self.mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
         self.mapping['id'] = uuid.uuid4().hex
-        self.federation_api.create_mapping(self.mapping['id'],
-                                           self.mapping)
+        PROVIDERS.federation_api.create_mapping(
+            self.mapping['id'], self.mapping
+        )
         protocol = {
             'id': uuid.uuid4().hex,
             'mapping_id': self.mapping['id']
         }
-        protocol_ret = self.federation_api.create_protocol(self.idp['id'],
-                                                           protocol['id'],
-                                                           protocol)
+        protocol_ret = PROVIDERS.federation_api.create_protocol(
+            self.idp['id'], protocol['id'], protocol
+        )
         try:
-            self.federation_api.create_protocol(self.idp['id'],
-                                                protocol['id'],
-                                                protocol)
+            PROVIDERS.federation_api.create_protocol(
+                self.idp['id'], protocol['id'], protocol
+            )
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % protocol_ret['id'], repr(e))
@@ -257,9 +274,9 @@ class DuplicateTestCase(test_v3.RestfulTestCase):
             'sp_url': uuid.uuid4().hex,
             'relay_state_prefix': CONF.saml.relay_state_prefix,
         }
-        service_ref = self.federation_api.create_sp('SP1', sp)
+        service_ref = PROVIDERS.federation_api.create_sp('SP1', sp)
         try:
-            self.federation_api.create_sp('SP1', sp)
+            PROVIDERS.federation_api.create_sp('SP1', sp)
         except exception.Conflict as e:
             self.assertIn("Duplicate entry found with ID %s"
                           % service_ref['id'], repr(e))
