@@ -15,14 +15,15 @@ import functools
 import os
 
 import fixtures
-from oslo_config import cfg
 from oslo_db import options as db_options
+from oslo_db.sqlalchemy import enginefacade
 
 from keystone.common import sql
+import keystone.conf
 from keystone.tests import unit
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 def run_once(f):
@@ -32,9 +33,9 @@ def run_once(f):
 
     """
     @functools.wraps(f)
-    def wrapper(one):
+    def wrapper():
         if not wrapper.already_ran:
-            f(one)
+            f()
             wrapper.already_ran = True
     wrapper.already_ran = False
     return wrapper
@@ -42,17 +43,25 @@ def run_once(f):
 
 # NOTE(I159): Every execution all the options will be cleared. The method must
 # be called at the every fixture initialization.
-def initialize_sql_session():
+def initialize_sql_session(connection_str=unit.IN_MEM_DB_CONN_STRING,
+                           enforce_sqlite_fks=True):
     # Make sure the DB is located in the correct location, in this case set
     # the default value, as this should be able to be overridden in some
     # test cases.
     db_options.set_defaults(
         CONF,
-        connection=unit.IN_MEM_DB_CONN_STRING)
+        connection=connection_str)
+
+    # Enable the  Sqlite FKs for global engine by default.
+    facade = enginefacade.writer
+    engine = facade.get_engine()
+    f_key = 'ON' if enforce_sqlite_fks else 'OFF'
+    if engine.name == 'sqlite':
+        engine.connect().execute('PRAGMA foreign_keys = ' + f_key)
 
 
 @run_once
-def _load_sqlalchemy_models(version_specifiers):
+def _load_sqlalchemy_models():
     """Find all modules containing SQLAlchemy models and import them.
 
     This creates more consistent, deterministic test runs because tables
@@ -67,24 +76,6 @@ def _load_sqlalchemy_models(version_specifiers):
     as more models are imported. Importing all models at the start of
     the test run avoids this problem.
 
-    version_specifiers is a dict that contains any specific driver versions
-    that have been requested. The dict is of the form:
-
-    {<module_name> : {'versioned_backend' : <name of backend requested>,
-                      'versionless_backend' : <name of default backend>}
-    }
-
-    For example:
-
-    {'keystone.assignment': {'versioned_backend' : 'V8_backends',
-                              'versionless_backend' : 'backends'},
-     'keystone.identity': {'versioned_backend' : 'V9_backends',
-                           'versionless_backend' : 'backends'}
-    }
-
-    The version_specifiers will be used to load the correct driver. The
-    algorithm for this assumes that versioned drivers begin in 'V'.
-
     """
     keystone_root = os.path.normpath(os.path.join(
         os.path.dirname(__file__), '..', '..', '..'))
@@ -104,32 +95,6 @@ def _load_sqlalchemy_models(version_specifiers):
             for x in range(0, len(module_components) - 1):
                 module_without_backends += module_components[x] + '.'
             module_without_backends = module_without_backends.rstrip('.')
-            this_backend = module_components[len(module_components) - 1]
-
-            # At this point module_without_backends might be something like
-            # 'keystone.assignment', while this_backend might be something
-            # 'V8_backends'.
-
-            if module_without_backends.startswith('keystone.contrib'):
-                # All the sql modules have now been moved into the core tree
-                # so no point in loading these again here (and, in fact, doing
-                # so might break trying to load a versioned driver.
-                continue
-
-            if module_without_backends in version_specifiers:
-                # OK, so there is a request for a specific version of this one.
-                # We therefore should skip any other versioned backend as well
-                # as the non-versioned one.
-                version = version_specifiers[module_without_backends]
-                if ((this_backend != version['versioned_backend'] and
-                     this_backend.startswith('V')) or
-                        this_backend == version['versionless_backend']):
-                    continue
-            else:
-                # No versioned driver requested, so ignore any that are
-                # versioned
-                if this_backend.startswith('V'):
-                    continue
 
             module_name = module_root + '.sql'
             __import__(module_name)
@@ -138,12 +103,11 @@ def _load_sqlalchemy_models(version_specifiers):
 class Database(fixtures.Fixture):
     """A fixture for setting up and tearing down a database."""
 
-    def __init__(self, version_specifiers=None):
+    def __init__(self):
         super(Database, self).__init__()
         initialize_sql_session()
-        if version_specifiers is None:
-            version_specifiers = {}
-        _load_sqlalchemy_models(version_specifiers)
+        _load_sqlalchemy_models()
+        sql.enable_sqlite_foreign_key()
 
     def setUp(self):
         super(Database, self).setUp()
