@@ -14,8 +14,6 @@
 
 """Fixtures for Federation Mapping."""
 
-from six.moves import range, zip
-
 
 EMPLOYEE_GROUP_ID = "0cd5e9"
 CONTRACTOR_GROUP_ID = "85a868"
@@ -1325,6 +1323,32 @@ MAPPING_FOR_EPHEMERAL_USER = {
     ]
 }
 
+MAPPING_FOR_EPHEMERAL_USER_AND_GROUP_DOMAIN_NAME = {
+    'rules': [
+        {
+            'local': [
+                {
+                    'user': {
+                        'name': '{0}',
+                        'type': 'ephemeral'
+                    },
+                    'group': {
+                        'name': 'dummy',
+                        'domain': {
+                            'name': 'dummy'
+                        }
+                    }
+                }
+            ],
+            'remote': [
+                {
+                    'type': 'SSL_CLIENT_USER_NAME'
+                }
+            ]
+        }
+    ]
+}
+
 MAPPING_FOR_DEFAULT_EPHEMERAL_USER = {
     'rules': [
         {
@@ -1406,6 +1430,36 @@ MAPPING_BAD_LOCAL_SETUP = {
     ]
 }
 
+MAPPING_BAD_LOCAL_TYPE_USER_IN_ASSERTION = {
+    "rules": [
+        {
+            "local": [
+                {
+                    "user": {
+                        "name": "{0}",
+                        "groups": "{1}"
+                    }
+                }
+            ],
+            "remote": [
+                {
+                    "type": "openstack_user"
+                },
+                {
+                    "type": "openstack_groups"
+
+                },
+                {
+                    "type": "openstack_roles",
+                    "any_one_of": [
+                        "Admin"
+                    ]
+                }
+            ]
+        },
+    ]
+}
+
 MAPPING_GROUPS_WITH_EMAIL = {
     "rules": [
         {
@@ -1438,6 +1492,35 @@ MAPPING_GROUPS_WITH_EMAIL = {
     ]
 }
 
+
+MAPPING_GROUPS_DOMAIN_OF_USER = {
+    "rules": [
+        {
+            "local":
+            [
+                {
+                    "user":
+                        {
+                            "name": "{0}"
+                        }
+                },
+                {
+                    "groups": "{1}"
+                }
+            ],
+            "remote":
+            [
+                {
+                    "type": "openstack_user"
+                },
+                {
+                    "type": "openstack_groups"
+                }
+            ]
+        }
+    ]
+}
+
 EMPLOYEE_ASSERTION = {
     'Email': 'tim@example.com',
     'UserName': 'tbo',
@@ -1596,6 +1679,14 @@ GROUPS_ASSERTION_ONLY_ONE_GROUP = {
     'groups': 'ALL USERS'
 }
 
+GROUPS_DOMAIN_ASSERTION = {
+    'openstack_user': 'bwilliams',
+    'openstack_user_domain': 'default',
+    'openstack_roles': 'Admin',
+    'openstack_groups': 'JSON:{"name":"group1","domain":{"name":"xxx"}};'
+                        'JSON:{"name":"group2","domain":{"name":"yyy"}}'
+}
+
 MAPPING_UNICODE = {
     "rules": [
         {
