@@ -16,20 +16,19 @@ import copy
 import datetime
 import logging
 import os
+from unittest import mock
 import uuid
 
 import argparse
+import configparser
 import fixtures
 import freezegun
-import mock
+import http.client
 import oslo_config.fixture
 from oslo_db.sqlalchemy import migration
 from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_upgradecheck import upgradecheck
-from six.moves import configparser
-from six.moves import http_client
-from six.moves import range
 from testtools import matchers
 
 from keystone.cmd import cli
@@ -220,11 +219,13 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                   headers={'X-Auth-Token': r.headers['X-Subject-Token'],
                            'X-Subject-Token': token})
         admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
-        reader_role = PROVIDERS.role_api.get_role(self.bootstrap.reader_role_id)
-        member_role = PROVIDERS.role_api.get_role(self.bootstrap.member_role_id)
-        self.assertEqual(admin_role['options'], {})
-        self.assertEqual(member_role['options'], {})
-        self.assertEqual(reader_role['options'], {})
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertEqual(admin_role['options'], {'immutable': True})
+        self.assertEqual(member_role['options'], {'immutable': True})
+        self.assertEqual(reader_role['options'], {'immutable': True})
 
     def test_bootstrap_is_not_idempotent_when_password_does_change(self):
         # NOTE(lbragstad): Ensure bootstrap isn't idempotent when run with
@@ -275,7 +276,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 c.get('/v3/auth/tokens',
                       headers={'X-Auth-Token': r.headers['X-Subject-Token'],
                                'X-Subject-Token': token},
-                      expected_status_code=http_client.NOT_FOUND)
+                      expected_status_code=http.client.NOT_FOUND)
 
     def test_bootstrap_recovers_user(self):
         self._do_test_bootstrap(self.bootstrap)
@@ -298,19 +299,50 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 user_id,
                 self.bootstrap.password)
 
-    def test_bootstrap_with_immutable_roles(self):
+    def test_bootstrap_with_explicit_immutable_roles(self):
         CONF(args=['bootstrap',
                    '--bootstrap-password', uuid.uuid4().hex,
                    '--immutable-roles'],
              project='keystone')
         self._do_test_bootstrap(self.bootstrap)
         admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
-        reader_role = PROVIDERS.role_api.get_role(self.bootstrap.reader_role_id)
-        member_role = PROVIDERS.role_api.get_role(self.bootstrap.member_role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
         self.assertTrue(admin_role['options']['immutable'])
         self.assertTrue(member_role['options']['immutable'])
         self.assertTrue(reader_role['options']['immutable'])
 
+    def test_bootstrap_with_default_immutable_roles(self):
+        CONF(args=['bootstrap',
+                   '--bootstrap-password', uuid.uuid4().hex],
+             project='keystone')
+        self._do_test_bootstrap(self.bootstrap)
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertTrue(admin_role['options']['immutable'])
+        self.assertTrue(member_role['options']['immutable'])
+        self.assertTrue(reader_role['options']['immutable'])
+
+    def test_bootstrap_with_no_immutable_roles(self):
+        CONF(args=['bootstrap',
+                   '--bootstrap-password', uuid.uuid4().hex,
+                   '--no-immutable-roles'],
+             project='keystone')
+        self._do_test_bootstrap(self.bootstrap)
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(
+            self.bootstrap.member_role_id)
+        self.assertNotIn('immutable', admin_role['options'])
+        self.assertNotIn('immutable', member_role['options'])
+        self.assertNotIn('immutable', reader_role['options'])
+
     def test_bootstrap_with_ambiguous_role_names(self):
         # bootstrap system to create the default admin role
         self._do_test_bootstrap(self.bootstrap)
@@ -612,7 +644,7 @@ class CliDomainConfigSingleDomainTestCase(CliDomainConfigAllTestCase):
         # Now try and upload the settings in the configuration file for the
         # default domain
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             file_name = ('keystone.%s.conf' % self.default_domain['name'])
             error_msg = _(
@@ -637,7 +669,7 @@ class CliDomainConfigNoOptionsTestCase(CliDomainConfigAllTestCase):
 
     def test_config_upload(self):
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             mock_print.assert_has_calls(
                 [mock.call(
@@ -654,7 +686,7 @@ class CliDomainConfigTooManyOptionsTestCase(CliDomainConfigAllTestCase):
 
     def test_config_upload(self):
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             mock_print.assert_has_calls(
                 [mock.call(_('The --all option cannot be used with '
@@ -671,7 +703,7 @@ class CliDomainConfigInvalidDomainTestCase(CliDomainConfigAllTestCase):
 
     def test_config_upload(self):
         provider_api.ProviderAPIs._clear_registry_instances()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             file_name = 'keystone.%s.conf' % self.invalid_domain_name
             error_msg = (_(
@@ -1844,7 +1876,7 @@ class TestMappingEngineTester(unit.BaseTestCase):
         self.useFixture(fixtures.MockPatchObject(
             CONF, 'command', self.FakeConfCommand(self)))
         mapping_engine = cli.MappingEngineTester()
-        with mock.patch('six.moves.builtins.print') as mock_print:
+        with mock.patch('builtins.print') as mock_print:
             mapping_engine.main()
             self.assertEqual(mock_print.call_count, 3)
             call = mock_print.call_args_list[0]
