@@ -14,13 +14,14 @@
 
 import hashlib
 import json
+from unittest import mock
 import uuid
 
+import http.client
 from keystoneclient.contrib.ec2 import utils as ec2_utils
-import mock
 from oslo_db import exception as oslo_db_exception
-from six.moves import http_client, urllib
 from testtools import matchers
+import urllib
 
 from keystone.api import ec2tokens
 from keystone.common import provider_api
@@ -87,7 +88,7 @@ class CredentialBaseTestCase(test_v3.RestfulTestCase):
         r = self.post(
             '/ec2tokens',
             body={'ec2Credentials': sig_ref},
-            expected_status=http_client.OK)
+            expected_status=http.client.OK)
         self.assertValidTokenResponse(r)
         return r.result['token']
 
@@ -265,7 +266,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
 
     def test_update_credential_to_ec2_with_previously_set_project_id(self):
         """Call ``PATCH /credentials/{credential_id}``."""
@@ -340,7 +341,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
         # Try removing the trust
         del blob['trust_id']
         update_ref = {'blob': json.dumps(blob)}
@@ -348,7 +349,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
 
     def test_update_ec2_credential_change_app_cred_id(self):
         """Call ``PATCH /credentials/{credential_id}``."""
@@ -368,7 +369,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
         # Try removing the app cred
         del blob['app_cred_id']
         update_ref = {'blob': json.dumps(blob)}
@@ -376,7 +377,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
 
     def test_update_ec2_credential_change_access_token_id(self):
         """Call ``PATCH /credentials/{credential_id}``."""
@@ -396,7 +397,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
         # Try removing the access token
         del blob['access_token_id']
         update_ref = {'blob': json.dumps(blob)}
@@ -404,7 +405,7 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': credential_id},
             body={'credential': update_ref},
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=http.client.BAD_REQUEST)
 
     def test_delete_credential(self):
         """Call ``DELETE /credentials/{credential_id}``."""
@@ -459,7 +460,7 @@ class CredentialTestCase(CredentialBaseTestCase):
         # for conflict.
         self.post(
             '/credentials',
-            body={'credential': ref}, expected_status=http_client.CONFLICT)
+            body={'credential': ref}, expected_status=http.client.CONFLICT)
 
     def test_get_ec2_dict_blob(self):
         """Ensure non-JSON blob data is correctly converted."""
@@ -514,7 +515,7 @@ class CredentialTestCase(CredentialBaseTestCase):
         # Assert bad request status when missing project_id
         self.post(
             '/credentials',
-            body={'credential': ref}, expected_status=http_client.BAD_REQUEST)
+            body={'credential': ref}, expected_status=http.client.BAD_REQUEST)
 
     def test_create_ec2_credential_with_invalid_blob(self):
         """Test creating ec2 credential with invalid blob.
@@ -528,7 +529,7 @@ class CredentialTestCase(CredentialBaseTestCase):
         # Assert bad request status when request contains invalid blob
         response = self.post(
             '/credentials',
-            body={'credential': ref}, expected_status=http_client.BAD_REQUEST)
+            body={'credential': ref}, expected_status=http.client.BAD_REQUEST)
         self.assertValidErrorResponse(response)
 
     def test_create_credential_with_admin_token(self):
@@ -632,7 +633,7 @@ class TestCredentialTrustScoped(CredentialBaseTestCase):
             '/credentials',
             body={'credential': ref},
             token=token_id,
-            expected_status=http_client.CONFLICT)
+            expected_status=http.client.CONFLICT)
 
 
 class TestCredentialAppCreds(CredentialBaseTestCase):
@@ -706,7 +707,7 @@ class TestCredentialAppCreds(CredentialBaseTestCase):
             '/credentials',
             body={'credential': ref},
             token=token_id,
-            expected_status=http_client.CONFLICT)
+            expected_status=http.client.CONFLICT)
 
 
 class TestCredentialAccessToken(CredentialBaseTestCase):
@@ -798,7 +799,7 @@ class TestCredentialAccessToken(CredentialBaseTestCase):
 
         url = self._authorize_request_token(request_key)
         body = {'roles': [{'id': self.role_id}]}
-        resp = self.put(url, body=body, expected_status=http_client.OK)
+        resp = self.put(url, body=body, expected_status=http.client.OK)
         verifier = resp.result['token']['oauth_verifier']
 
         request_token.set_verifier(verifier)
@@ -921,7 +922,7 @@ class TestCredentialEc2(CredentialBaseTestCase):
         uri = '/'.join([self._get_ec2_cred_uri(), access_key])
         # if access_key is not found, ec2 controller raises Unauthorized
         # exception
-        self.get(uri, expected_status=http_client.UNAUTHORIZED)
+        self.get(uri, expected_status=http.client.UNAUTHORIZED)
 
     def test_ec2_list_credentials(self):
         """Test ec2 credential listing."""
